/**
 * Copyright (C) 2007 - Bull S.A.S.
 *
 * CAROL: Common Architecture for RMI ObjectWeb Layer
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ServerConfigurationImplMBean.java 1482 2007-11-14 15:19:59Z loris $
 * --------------------------------------------------------------------------
 */
package org.ow2.carol.util.configuration;

public interface ServerConfigurationImplMBean {

    /**
     * @return Gets the object name of this mbean
     */
    String getObjectName();

    /**
     * @return true if JNDI has to be started (set of MultiORB ICTX factory)
     */
    boolean isStartingJNDI();

    /**
     * @return true if name services have to be launched)
     */
    boolean isStartingNS();

    /**
     * @return true if ProdelegateClass has to be set to MultiProDelegate
     */
    boolean isStartingRMI();

    /* -------------- Configuration of CMI -----------------*/
    /**
     * @return true if CMI is used
     */
    boolean isStartCMI();

    /**
     * Set if CMI is used.
     * @param startCMI true if CMI is used
     * @throws ConfigurationException if cmi cannot be configured
     */
    void setStartCMI(boolean startCMI) throws Exception;

    /**
     * @return a InitialContextFactory name to use with CMI
     */
    String getCmiInitialContextFactory();

    /**
     * Set the object name of this mbean.
     * @param name the Object Name
     */
    void setObjectName(String string);

    /**
     * @return true if this is a server
     */
    boolean isServer();

    /**
     * return true if this server can register clustered objects (only for servers)
     */
    boolean isReplicationEnabled();

    /**
     * Set if this server can register clustered objects (only for servers).
     * @param replicationEnabled true if this server can register clustered objects (only for servers)
     * @throws Exception if cmi cannot be configured
     */
    void setReplicationEnabled(boolean replicationEnabled) throws Exception;

}
